@@ -410,28 +410,27 @@ bool print_alias_if_defined(const wchar_t *aliasname, bool user_friendly)
 
 #if YASH_ENABLE_LINEEDIT
 
-/* Generates candidates to complete an alias matching the pattern in the
- * specified context. */
+/* Generates candidates to complete an alias matching the pattern. */
 /* The prototype of this function is declared in "lineedit/complete.h". */
-void generate_alias_candidates(le_candgentype_T type, le_context_T *context)
+void generate_alias_candidates(const le_compopt_T *compopt)
 {
-    if (!(type & CGT_ALIAS))
+    if (!(compopt->type & CGT_ALIAS))
 	return;
 
-    le_compdebug("adding aliases matching pattern \"%ls\"", context->pattern);
-    if (!le_compile_cpattern(context))
+    le_compdebug("adding alias name candidates");
+    if (!le_compile_cpatterns(compopt))
 	return;
 
     size_t i = 0;
     kvpair_T kv;
     while ((kv = ht_next(&aliases, &i)).key != NULL) {
 	const alias_T *alias = kv.value;
-	if ((alias->flags & AF_GLOBAL)
-		? (type & CGT_GALIAS) : (type & CGT_NALIAS)) {
-	    if (xfnm_wmatch(context->cpattern, kv.key).start != (size_t) -1) {
-		le_new_candidate(CT_ALIAS, xwcsdup(kv.key), NULL);
-	    }
-	}
+	le_candgentype_T type =
+	    (alias->flags & AF_GLOBAL) ? CGT_GALIAS : CGT_NALIAS;
+
+	if (compopt->type & type)
+	    if (le_wmatch_comppatterns(compopt, kv.key))
+		le_new_candidate(CT_ALIAS, xwcsdup(kv.key), NULL, compopt);
     }
 }
 
@@ -446,10 +445,10 @@ void generate_alias_candidates(le_candgentype_T type, le_context_T *context)
 int alias_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"global", xno_argument, L'g', },
-	{ L"prefix", xno_argument, L'p', },
+	{ L"global", OPTARG_NONE, L'g', },
+	{ L"prefix", OPTARG_NONE, L'p', },
 #if YASH_ENABLE_HELP
-	{ L"help",   xno_argument, L'-', },
+	{ L"help",   OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
