@@ -23,13 +23,6 @@
 
 #include "config.h"
 
-#ifndef _POSIX_C_SOURCE
-# define _POSIX_C_SOURCE 200112L
-#endif
-#ifndef _XOPEN_SOURCE
-# define _XOPEN_SOURCE 600
-#endif
-
 #ifndef __GNUC__
 # define __attribute__(ignore)
 #endif
