@@ -80,6 +80,7 @@ static bool ctryparse_assignment(void);
 static bool ctryparse_redirect(void);
 static bool cparse_for_command(void);
 static bool cparse_case_command(void);
+static bool cparse_function_definition(void);
 static wchar_t *cparse_and_expand_word(
 	tildetype_T tilde, le_contexttype_T ctxttype)
     __attribute__((malloc,warn_unused_result));
@@ -116,8 +117,7 @@ static bool remove_braceexpand_inner(wchar_t **s)
 /* Parses the contents of the edit buffer (`le_main_buffer') from the beginning
  * up to the current cursor position (`le_main_index') and determines the
  * current completion context.
- * The results are returned as a newly malloced `le_context_T' data, but the
- * `cpattern' member is left NULL. */
+ * The results are returned as a newly malloced `le_context_T' data. */
 le_context_T *le_get_context(void)
 {
     assert(wcslen(le_main_buffer.contents) == le_main_buffer.length);
@@ -148,7 +148,7 @@ le_context_T *le_get_context(void)
     if (shopt_braceexpand)
 	if (remove_braceexpand(ctxt->pattern))
 	    ctxt->type |= CTXT_EBRACED;
-    ctxt->src = ctxt->origsrc = unescape(ctxt->pattern);
+    ctxt->src = unescape(ctxt->pattern);
     if (is_pathname_matching_pattern(ctxt->pattern)) {
 	ctxt->substsrc = true;
     } else {
@@ -157,8 +157,6 @@ le_context_T *le_get_context(void)
 	ctxt->pattern =
 	    wb_towcs(wb_wccat(wb_initwith(&buf, ctxt->pattern), L'*'));
     }
-    ctxt->origpattern = ctxt->pattern;
-    ctxt->cpattern = NULL;
     ctxt->origindex = le_main_index;
 
     return ctxt;
@@ -265,6 +263,8 @@ bool cparse_command(void)
 	return cparse_for_command();
     } else if (token_at_current(L"case")) {
 	return cparse_case_command();
+    } else if (!posixly_correct && token_at_current(L"function")) {
+	return cparse_function_definition();
     } else {
 	return cparse_simple_command();
     }
@@ -312,7 +312,7 @@ cparse_simple_command:
 	}
 
 	wordunit_T *w = cparse_word(is_token_delimiter_char, tt_single,
-		pwords.length == 0 ? CTXT_COMMAND : CTXT_NORMAL);
+		pwords.length == 0 ? CTXT_COMMAND : CTXT_ARGUMENT);
 	if (w == NULL) {
 	    if (pi->ctxt->pwords == NULL) {
 		pi->ctxt->pwordc = pwords.length;
@@ -586,6 +586,26 @@ bool cparse_case_command(void)
     return false;
 }
 
+/* Parses a function definition command.
+ * There must be the "function" keyword at the current position. */
+bool cparse_function_definition(void)
+{
+    assert(wcsncmp(BUF + INDEX, L"function", 8) == 0);
+    INDEX += 8;
+    skip_blanks();
+
+    /* parse the function name */
+    wordunit_T *w =
+	cparse_word(is_token_delimiter_char, tt_none, CTXT_FUNCTION);
+    if (w == NULL) {
+	empty_pwords();
+	return true;
+    }
+    wordfree(w);
+
+    return false;
+}
+
 /* Parses the word at the current position.
  * `skip_blanks' should be called before this function is called.
  * The `ctxttype' parameter is the context type of the word parsed.
