@@ -334,23 +334,22 @@ void le_set_mode(le_mode_id_T id)
     le_current_mode = le_id_to_mode(id);
 }
 
-/* Generates completion candidates for editing command names that match the glob
- * pattern in the specified context. */
+/* Generates completion candidates for editing command names matching the
+ * pattern. */
 /* The prototype of this function is declared in "complete.h". */
-void generate_bindkey_candidates(le_candgentype_T type, le_context_T *context)
+void generate_bindkey_candidates(const le_compopt_T *compopt)
 {
-    if (!(type & CGT_BINDKEY))
+    if (!(compopt->type & CGT_BINDKEY))
 	return;
 
-    le_compdebug("adding lineedit commands for pattern \"%ls\"",
-	    context->pattern);
-    if (!le_compile_cpattern(context))
+    le_compdebug("adding lineedit command name candidates");
+    if (!le_compile_cpatterns(compopt))
 	return;
 
     for (size_t i = 0; i < sizeof commands / sizeof *commands; i++)
-	if (xfnm_match(context->cpattern, commands[i].name) == 0)
-	    le_new_candidate(
-		    CT_BINDKEY, malloc_mbstowcs(commands[i].name), NULL);
+	if (le_match_comppatterns(compopt, commands[i].name))
+	    le_new_candidate(CT_BINDKEY,
+		    malloc_mbstowcs(commands[i].name), NULL, compopt);
 }
 
 
@@ -380,12 +379,12 @@ static const char *get_command_name(le_command_func_T *command)
 int bindkey_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"vi-insert",  xno_argument, L'v', },
-	{ L"vi-command", xno_argument, L'a', },
-	{ L"emacs",      xno_argument, L'e', },
-	{ L"list",       xno_argument, L'l', },
+	{ L"vi-insert",  OPTARG_NONE, L'v', },
+	{ L"vi-command", OPTARG_NONE, L'a', },
+	{ L"emacs",      OPTARG_NONE, L'e', },
+	{ L"list",       OPTARG_NONE, L'l', },
 #if YASH_ENABLE_HELP
-	{ L"help",       xno_argument, L'-', },
+	{ L"help",       OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
