@@ -24,13 +24,14 @@
 
 
 enum le_state_T {
-    LE_STATE_INACTIVE            = (0 << 1) | 0,
-    LE_STATE_SUSPENDED           = (1 << 1) | 0,
-    LE_STATE_SUSPENDED_COMPDEBUG = (2 << 1) | 0,
-    LE_STATE_ACTIVE              = (0 << 1) | 1,
-    LE_STATE_ACTIVE_COMPLETING   = (1 << 1) | 1,
+    LE_STATE_INACTIVE   = 0,
+    LE_STATE_ACTIVE     = 1 << 0,
+    LE_STATE_SUSPENDED  = 1 << 1,
+    LE_STATE_COMPLETING = 1 << 2,
 };
-#define le_state_is_active (le_state & 1)
+#define le_state_is_compdebug \
+    ((le_state & (LE_STATE_SUSPENDED | LE_STATE_COMPLETING)) \
+              == (LE_STATE_SUSPENDED | LE_STATE_COMPLETING))
 
 enum le_editstate_T {
     LE_EDITSTATE_EDITING,     // editing is on-going
