@@ -636,7 +636,7 @@ inputresult_T read_more_input(void)
 
 	if (cinfo->enable_verbose && shopt_verbose)
 #if YASH_ENABLE_LINEEDIT
-	    if (!le_state_is_active)
+	    if (!(le_state & LE_STATE_ACTIVE))
 #endif
 		fprintf(stderr, "%ls", cbuf.contents + savelength);
     }
