@@ -644,8 +644,6 @@ static struct wglob_pattern *wglob_parse_pattern_part(
 	wchar_t *pat, enum wglbflags flags)
     __attribute__((malloc,warn_unused_result,nonnull));
 static void wglob_free_pattern(struct wglob_pattern *p);
-static int wglob_sortcmp(const void *v1, const void *v2)
-    __attribute__((pure,nonnull));
 static void wglob_search(
 	const struct wglob_pattern *restrict pattern,
 	enum wglbflags flags,
@@ -678,6 +676,8 @@ static void wglob_search_recsearch(
 static bool is_reentry(
 	const struct stat *st, const struct wglob_dirstack *dirstack)
     __attribute__((nonnull(1)));
+static int wglob_sortcmp(const void *v1, const void *v2)
+    __attribute__((pure,nonnull));
 
 /* A wide string version of `glob'.
  * Adds all pathnames that matches the specified pattern to the specified list.
@@ -843,11 +843,6 @@ void wglob_free_pattern(struct wglob_pattern *p)
     }
 }
 
-int wglob_sortcmp(const void *v1, const void *v2)
-{
-    return wcscoll(*(const wchar_t *const *) v1, *(const wchar_t *const *) v2);
-}
-
 /* Searches the directory designated in `path' and add matching pathnames to
  * `list'.
  * `path' and `wpath' must contain the same pathname, which must be empty or
@@ -1022,6 +1017,11 @@ bool is_reentry(const struct stat *st, const struct wglob_dirstack *dirstack)
     return false;
 }
 
+int wglob_sortcmp(const void *v1, const void *v2)
+{
+    return wcscoll(*(const wchar_t *const *) v1, *(const wchar_t *const *) v2);
+}
+
 
 /********** Builtins **********/
 
@@ -1395,11 +1395,11 @@ const char pwd_help[] = Ngt(
 int hash_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"all",       xno_argument, L'a', },
-	{ L"directory", xno_argument, L'd', },
-	{ L"remove",    xno_argument, L'r', },
+	{ L"all",       OPTARG_NONE, L'a', },
+	{ L"directory", OPTARG_NONE, L'd', },
+	{ L"remove",    OPTARG_NONE, L'r', },
 #if YASH_ENABLE_HELP
-	{ L"help",      xno_argument, L'-', },
+	{ L"help",      OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
@@ -1548,9 +1548,9 @@ const char hash_help[] = Ngt(
 int umask_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"symbolic", xno_argument, L'S', },
+	{ L"symbolic", OPTARG_NONE, L'S', },
 #if YASH_ENABLE_HELP
-	{ L"help",     xno_argument, L'-', },
+	{ L"help",     OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
