@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* redir.h: manages file descriptors and provides functions for redirections */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ extern _Bool is_shellfd(int fd)
     __attribute__((pure));
 extern void clear_shellfds(_Bool leavefds);
 extern int copy_as_shellfd(int fd);
-extern FILE *reopen_with_shellfd(FILE *f, const char *mode, _Bool nonblock);
+static inline int move_to_shellfd(int fd);
 extern void open_ttyfd(void);
 extern int get_ttyfd(void) __attribute__((pure));
 
@@ -52,6 +52,21 @@ extern void maybe_redirect_stdin_to_devnull(void);
 #define PIDX_OUT 1   /* index of the writing end of a pipe */
 
 
+/* Moves the specified file descriptor (FD) to a shell FD.
+ * The original FD is closed (whether successful or not).
+ * If `fd' is negative, this function simply returns `fd' (without changing
+ * `errno'). */
+int move_to_shellfd(int fd)
+{
+    if (fd < 0)
+	return fd;
+
+    int newfd = copy_as_shellfd(fd);
+    xclose(fd);
+    return newfd;
+}
+
+
 #endif /* YASH_REDIR_H */
 
 
