@@ -60,9 +60,7 @@
 static const wchar_t *const path_variables[PA_count] = {
     [PA_PATH]     = L VAR_PATH,
     [PA_CDPATH]   = L VAR_CDPATH,
-#if YASH_ENABLE_LINEEDIT
-    [PA_COMPPATH] = L VAR_YASH_COMPPATH,
-#endif
+    [PA_LOADPATH] = L VAR_YASH_LOADPATH,
 };
 
 
@@ -84,7 +82,7 @@ typedef struct environ_T {
 /* An environment whose `is_temporary' is true is used for temporary variables. 
  * Temporary variables may be exported but cannot be readonly.
  * The elements of `paths' are arrays of the pathnames contained in the
- * $PATH, $CDPATH and $YASH_COMPPATH variables. They are NULL if the
+ * $PATH, $CDPATH and $YASH_LOADPATH variables. They are NULL if the
  * corresponding variables are not set. */
 #define VAR_positional "="
 
@@ -331,8 +329,8 @@ void init_variables(void)
 	random_active = false;
     }
 
-    /* set $YASH_COMPPATH */
-    set_variable(L VAR_YASH_COMPPATH, xwcsdup(L YASH_DATADIR "/completion"),
+    /* set $YASH_LOADPATH */
+    set_variable(L VAR_YASH_LOADPATH, xwcsdup(L YASH_DATADIR),
 	    SCOPE_GLOBAL, false);
 
     /* set $YASH_VERSION */
@@ -1042,11 +1040,11 @@ void variable_set(const wchar_t *name, variable_T *var)
 	if (wcscmp(name, L VAR_TERM) == 0)
 	    le_need_term_update = true;
 	break;
-    case L'Y':
-	if (wcscmp(name, L VAR_YASH_COMPPATH) == 0)
-	    reset_path(PA_COMPPATH, var);
-	break;
 #endif /* YASH_ENABLE_LINEEDIT */
+    case L'Y':
+	if (wcscmp(name, L VAR_YASH_LOADPATH) == 0)
+	    reset_path(PA_LOADPATH, var);
+	break;
     }
 }
 
@@ -1273,16 +1271,15 @@ void tryhash_word_as_command(const wordunit_T *w)
 
 #if YASH_ENABLE_LINEEDIT
 
-/* Generates completion candidates for variable names that match the glob
- * pattern in the specified context. */
+/* Generates completion candidates for variable names matching the pattern. */
 /* The prototype of this function is declared in "lineedit/complete.h". */
-void generate_variable_candidates(le_candgentype_T type, le_context_T *context)
+void generate_variable_candidates(const le_compopt_T *compopt)
 {
-    if (!(type & CGT_VARIABLE))
+    if (!(compopt->type & CGT_VARIABLE))
 	return;
 
-    le_compdebug("adding variables for pattern \"%ls\"", context->pattern);
-    if (!le_compile_cpattern(context))
+    le_compdebug("adding variable name candidates");
+    if (!le_compile_cpatterns(compopt))
 	return;
 
     size_t i = 0;
@@ -1291,32 +1288,36 @@ void generate_variable_candidates(le_candgentype_T type, le_context_T *context)
 	const wchar_t *name = kv.key;
 	const variable_T *var = kv.value;
 	switch (var->v_type & VF_MASK) {
-	    case VF_SCALAR:  if (!(type & CGT_SCALAR)) continue;  break;
-	    case VF_ARRAY:   if (!(type & CGT_ARRAY))  continue;  break;
+	    case VF_SCALAR:
+		if (!(compopt->type & CGT_SCALAR))
+		    continue;
+		break;
+	    case VF_ARRAY:
+		if (!(compopt->type & CGT_ARRAY))
+		    continue;
+		break;
 	}
-	if (name[0] != L'='
-		&& xfnm_wmatch(context->cpattern, name).start != (size_t) -1)
-	    le_new_candidate(CT_VAR, xwcsdup(name), NULL);
+	if (name[0] != L'=' && le_wmatch_comppatterns(compopt, name))
+	    le_new_candidate(CT_VAR, xwcsdup(name), NULL, compopt);
     }
 }
 
-/* Generates completion candidates for function names that match the glob
- * pattern in the specified context. */
+/* Generates completion candidates for function names matching the pattern. */
 /* The prototype of this function is declared in "lineedit/complete.h". */
-void generate_function_candidates(le_candgentype_T type, le_context_T *context)
+void generate_function_candidates(const le_compopt_T *compopt)
 {
-    if (!(type & CGT_FUNCTION))
+    if (!(compopt->type & CGT_FUNCTION))
 	return;
 
-    le_compdebug("adding functions for pattern \"%ls\"", context->pattern);
-    if (!le_compile_cpattern(context))
+    le_compdebug("adding function name candidates");
+    if (!le_compile_cpatterns(compopt))
 	return;
 
     size_t i = 0;
     const wchar_t *name;
     while ((name = ht_next(&functions, &i).key) != NULL)
-	if (xfnm_wmatch(context->cpattern, name).start != (size_t) -1)
-	    le_new_command_candidate(xwcsdup(name));
+	if (le_wmatch_comppatterns(compopt, name))
+	    le_new_candidate(CT_COMMAND, xwcsdup(name), NULL, compopt);
 }
 
 #endif /* YASH_ENABLE_LINEEDIT */
@@ -1463,14 +1464,14 @@ static void split_and_assign_array(const wchar_t *name, wchar_t *values,
 int typeset_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"functions", xno_argument, L'f', },
-	{ L"global",    xno_argument, L'g', },
-	{ L"print",     xno_argument, L'p', },
-	{ L"readonly",  xno_argument, L'r', },
-	{ L"export",    xno_argument, L'x', },
-	{ L"unexport",  xno_argument, L'X', },
+	{ L"functions", OPTARG_NONE, L'f', },
+	{ L"global",    OPTARG_NONE, L'g', },
+	{ L"print",     OPTARG_NONE, L'p', },
+	{ L"readonly",  OPTARG_NONE, L'r', },
+	{ L"export",    OPTARG_NONE, L'x', },
+	{ L"unexport",  OPTARG_NONE, L'X', },
 #if YASH_ENABLE_HELP
-	{ L"help",      xno_argument, L'-', },
+	{ L"help",      OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
@@ -1802,11 +1803,11 @@ const char typeset_help[] = Ngt(
 int array_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"delete", xno_argument, L'd', },
-	{ L"insert", xno_argument, L'i', },
-	{ L"set",    xno_argument, L's', },
+	{ L"delete", OPTARG_NONE, L'd', },
+	{ L"insert", OPTARG_NONE, L'i', },
+	{ L"set",    OPTARG_NONE, L's', },
 #if YASH_ENABLE_HELP
-	{ L"help",   xno_argument, L'-', },
+	{ L"help",   OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
@@ -2074,10 +2075,10 @@ const char array_help[] = Ngt(
 int unset_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"functions", xno_argument, L'f', },
-	{ L"variables", xno_argument, L'v', },
+	{ L"functions", OPTARG_NONE, L'f', },
+	{ L"variables", OPTARG_NONE, L'v', },
 #if YASH_ENABLE_HELP
-	{ L"help",      xno_argument, L'-', },
+	{ L"help",      OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
@@ -2482,10 +2483,10 @@ const char getopts_help[] = Ngt(
 int read_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"array",    xno_argument, L'A', },
-	{ L"raw-mode", xno_argument, L'r', },
+	{ L"array",    OPTARG_NONE, L'A', },
+	{ L"raw-mode", OPTARG_NONE, L'r', },
 #if YASH_ENABLE_HELP
-	{ L"help",     xno_argument, L'-', },
+	{ L"help",     OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
@@ -2702,11 +2703,11 @@ const char read_help[] = Ngt(
 
 /* options for the "pushd" builtins */
 static const struct xoption pushd_options[] = {
-    { L"default-directory", xrequired_argument, L'd', },
-    { L"logical",           xno_argument,       L'L', },
-    { L"physical",          xno_argument,       L'P', },
+    { L"default-directory", OPTARG_REQUIRED, L'd', },
+    { L"logical",           OPTARG_NONE,     L'L', },
+    { L"physical",          OPTARG_NONE,     L'P', },
 #if YASH_ENABLE_HELP
-    { L"help",              xno_argument,       L'-', },
+    { L"help",              OPTARG_NONE,     L'-', },
 #endif
     { NULL, 0, 0, },
 };
@@ -2719,12 +2720,12 @@ const struct xoption *const pwd_options = pushd_options + 1;
 
 /* options for the "pushd" builtins */
 static const struct xoption pushd_options[] = {
-    { L"remove-duplicates", xno_argument,       L'D', },
-    { L"default-directory", xrequired_argument, L'd', },
-    { L"logical",           xno_argument,       L'L', },
-    { L"physical",          xno_argument,       L'P', },
+    { L"remove-duplicates", OPTARG_NONE,     L'D', },
+    { L"default-directory", OPTARG_REQUIRED, L'd', },
+    { L"logical",           OPTARG_NONE,     L'L', },
+    { L"physical",          OPTARG_NONE,     L'P', },
 #if YASH_ENABLE_HELP
-    { L"help",              xno_argument,       L'-', },
+    { L"help",              OPTARG_NONE,     L'-', },
 #endif
     { NULL, 0, 0, },
 };
@@ -2981,10 +2982,10 @@ const char popd_help[] = Ngt(
 int dirs_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
-	{ L"clear",   xno_argument, L'c', },
-	{ L"verbose", xno_argument, L'v', },
+	{ L"clear",   OPTARG_NONE, L'c', },
+	{ L"verbose", OPTARG_NONE, L'v', },
 #if YASH_ENABLE_HELP
-	{ L"help",    xno_argument, L'-', },
+	{ L"help",    OPTARG_NONE, L'-', },
 #endif
 	{ NULL, 0, 0, },
     };
