@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* yash.h: basic functions of the shell and miscellanies */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2010 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 #ifndef YASH_YASH_H
 #define YASH_YASH_H
 
-#include <stdio.h>
 #include <sys/types.h>
 
 
@@ -31,15 +30,13 @@ extern void exit_shell_with_status(int status)
     __attribute__((noreturn));
 
 
-struct parseinfo_T;
+extern struct input_file_info *stdin_input_file_info;
 
+struct parseinfo_T;
 extern void exec_wcs(const wchar_t *code, const char *name, _Bool finally_exit)
     __attribute__((nonnull(1)));
-extern void exec_input(FILE *f, const char *name,
-	_Bool intrinput, _Bool enable_alias, _Bool finally_exit)
-    __attribute__((nonnull(1)));
-extern void parse_and_exec(struct parseinfo_T *pinfo, _Bool finally_exit)
-    __attribute__((nonnull(1)));
+extern void exec_input(int fd, const char *name,
+	_Bool intrinput, _Bool enable_alias, _Bool finally_exit);
 
 
 extern _Bool nextforceexit;
